import time
from math import floor

import re
from pytimeparse.timeparse import timeparse

import datetime

from swiss_common_utils.utils.datetime.exceptions.NotATimeStringError import NotATimeStringError
from swiss_common_utils.utils.json.json_utils import beautify_json
from swiss_common_utils.utils.log.log_utils import get_logger

logger = get_logger()


def get_current_time_millis():
    return int(round(time.time() * 1000))


def get_last_hour_in_millis(hours=1):
    now_millis = int(datetime.datetime.now().timestamp()) * 1000
    hours_in_millis = hours_to_millis(hours)
    return now_millis - hours_in_millis


def get_last_minute_in_millis(minutes=0):
    now_millis = int(datetime.datetime.now().timestamp()) * 1000
    minutes_in_millis = minutes_to_millis(minutes)
    return now_millis - minutes_in_millis


def seconds_to_millis(seconds):
    return seconds * 1000


def minutes_to_millis(minutes):
    return seconds_to_millis(minutes * 60)


def hours_to_millis(hours):
    return minutes_to_millis(hours * 60)


def millis_to_datetime(millis):
    return datetime.datetime.fromtimestamp(millis / 1000.0)


def beautify_datetime(date_time):
    date_time_dict = {'Date': str(date_time.date()), 'Time': str(date_time.time().replace(microsecond=0))}
    date_time_pretty = beautify_json(date_time_dict)
    return date_time_pretty


def datetime_to_millis(dt):
    epoch = datetime.datetime.utcfromtimestamp(0)
    return floor((dt - epoch).total_seconds() * 1000)


def time_string_to_seconds(time_str):
    if not bool(re.match('^\d+(\.*\d+)*[smhdw]$', time_str)):
        if bool(re.match('^\d+(\.*\d+)*$', time_str)):
            global logger
            logger.warning('Time category missing (s\m\h\d\w). Default is seconds -> {}s'.format(time_str))
            return timeparse(time_str + 's')
        raise NotATimeStringError(
            'Failed to interpert "{}" as time string. Expression should match the following regex: \d+(\.*\d+)*[smhdw]'.format(
                time_str))

    return timeparse(time_str)


def format_time_length(milliseconds=None, seconds=None, minutes=None, hours=None):
    secs = 0
    if milliseconds:
        secs = milliseconds / 1000
    elif seconds:
        secs = seconds
    elif minutes:
        secs = minutes * 60
    elif hours:
        secs = hours * 3600

    return time.strftime('%Hh:%Mm:%Ss', time.gmtime(secs))



import io

from memory_profiler import profile

memory_usage_stream = io.StringIO('')


def record_memory_usage(activate=False, stream=None):
    def wrap(org_func):
        if activate:

            @profile(stream=stream)
            def wrapped_func(*args, **kwargs):
                return org_func(*args, **kwargs)
            return wrapped_func
        else:
            def wrapped_func(*args, **kwargs):
                return org_func(*args, **kwargs)
            return wrapped_func

    return wrap



import collections
import yaml


from swiss_common_utils.utils.log.log_utils import get_logger


def flatten_dict(d, parent_key='', sep='_', to_upper=False):
    items = []
    for k, v in d.items():
        new_key = parent_key + sep + k if parent_key else k
        if to_upper:
            new_key = new_key.upper()
        if isinstance(v, collections.MutableMapping):
            items.extend(flatten_dict(v, new_key, sep=sep, to_upper=to_upper).items())
        else:
            items.append((new_key, v))
    return dict(items)


def yaml_to_flat_dict(src_file):
    logger = get_logger()
    try:
        f_yaml = open(src_file, 'r')
        d = yaml.load(f_yaml)
        return flatten_dict(d=d)

    except Exception as e:
        logger.error('Exception caught. type: {}, message: {}'.format(type(e), str(e)))
        return {}


import json
import requests

from swiss_common_utils.utils.json.json_utils import beautify_json
from swiss_common_utils.utils.log.log_utils import get_logger
from swiss_common_utils.utils.network.url.url_utils import add_http_if_missing


class SWISSServiceBase:
    logger = get_logger()

    HEADERS = {'Content-type': 'application/json'}

    def __init__(self, host, port=None, log_response=False):
        self._host = add_http_if_missing(host)
        self._port = port
        self._log_response = log_response

    @property
    def host(self):
        return self._host

    @host.setter
    def host(self, host):
        self._host = host

    @property
    def port(self):
        return self._port

    @port.setter
    def port(self, port):
        self._port = port

    def _get_url(self, path_params_list=None, request_params_dict=None):
        url = self.host
        if self.port:
            url = url + ':' + self.port
        if path_params_list is not None:
            url = url.strip('/') + '/' + self.__build_url_path(path_params_list)
        if request_params_dict is not None:
            url = url + '?' + self.__build_url_parameters(request_params_dict)

        return url

    def __handle_response_code(self, response_code):
        self.logger.info('HTTP response code: ' + str(response_code))
        if response_code != 200:
            self.logger.error('HTTP response code description: ' + requests.status_codes._codes[response_code][0])

    def _handle_response(self, response):
        response_code = response.status_code

        self.__handle_response_code(response_code)

        response_elapsed_milli = int(round(response.elapsed.total_seconds() * 1000))
        self.logger.info('Request elapsed: ' + str(response_elapsed_milli) + 'ms')
        self.logger.info('Response headers: {}'.format(response.headers))
        response_content = response.content.decode('utf-8')
        response_content_json = json.loads(response_content)
        if self._log_response:
            self.logger.info('Response: \n' + beautify_json(response_content_json))
        return response_content_json

    def __build_url_path(self, path_params_list):
        return '/'.join(path_params_list)

    def __build_url_parameters(self, url_params_dict):
        key_val_list = []
        for key in url_params_dict:
            key_val_list.append(key + '=' + url_params_dict[key])

        return '&'.join(key_val_list)

    def dispatch(self, url, data=None):
        self.logger.info('### Sending request: ###')
        self.logger.info('URL: ' + url)

        # TODO: requests go through the proxy because of the env var. should find a workaround
        if data is None:
            self.logger.info('Method: GET')

            return requests.get(url=url, headers=self.HEADERS)
        else:
            self.logger.info('Method: POST')
            self.logger.info('Post data: {}'.format(data))
            return requests.post(url=url, headers=self.HEADERS, data=data)



import json


def beautify_json(json_obj):
    return json.dumps(json_obj, indent=2, sort_keys=True)


def print_pretty(data):
    if type(data) is str \
            or type(data) is dict \
            or type(data) is list:
        json_obj = data
        if type(data) is str:
            json_obj = json.loads(json)
        json_str = beautify_json(json_obj)
        print(json_str)
    else:
        print('print pretty failed')


def flatten_json(y):
    out = {}

    def flatten(x, name=''):
        if type(x) is dict:
            for a in x:
                flatten(x[a], name + a + '.')
        elif type(x) is list:
            i = 0
            for a in x:
                flatten(a, name + str(i) + '.')
                i += 1
        else:
            out[name[:-1]] = x

    flatten(y)
    return out



def starts_with_http(str):
    return str.startswith('http://')


def starts_with_https(str):
    return str.startswith('https://')


def add_http_if_missing(url):
    if not starts_with_http(url) and not starts_with_https(url):
        url = 'http://' + url
    return url



from setuptools import setup, find_packages

from swiss_common_utils.install_requirements import get_required_packages
from swiss_common_utils.version import get_package_version

setup(
    name='swiss_common_utils',
    version=get_package_version(),
    packages=find_packages(),
    url='',
    license='',
    author='lior.yehonatan',
    author_email='lior.yehonatan@inel.com',
    description='swiss common utils',
    python_requires='>=3.6',
    install_requires=get_required_packages()
)



import logging
import time

import coloredlogs
import pyfiglet

logger = None


def get_logger(level='DEBUG', filename=None):
    global logger
    if logger:
        return logger

    logging.basicConfig(format='%(asctime)s %(funcName)s():%(lineno)i:%(message)s', datefmt='%m/%d/%Y %I:%M:%S %p',
                        filename=filename)
    logger = logging.getLogger()
    coloredlogs.install(level=level)
    return logger


def log_enter_and_exit(orig_func):
    logger = get_logger()

    def wrapper(*args, **kwargs):
        func_full_name = '.'.join([orig_func.__module__, orig_func.__qualname__])
        logger.debug('[ENTER] {}'.format(func_full_name))
        start = int(time.time()) * 1000
        result = orig_func(*args, **kwargs)
        run_time = (int(time.time()) * 1000) - start
        logger.debug('[EXIT] {} (Execution time: {}ms)'.format(func_full_name, run_time))
        return result

    return wrapper


def log_function_args_and_kwargs(orig_func):
    logger = get_logger()

    def wrapper(*args, **kwargs):
        logger.info('Function: {} run with args: {}, and kwargs: {}'.format(orig_func.__name__, args, kwargs))
        return orig_func(*args, **kwargs)

    return wrapper


exec_time_record = []


def log_function_runtime(orig_func):
    logger = get_logger()
    import time

    def wrapper(*args, **kwargs):
        start = int(time.time()) * 1000
        result = orig_func(*args, **kwargs)
        run_time = (int(time.time()) * 1000) - start
        logger.info('Function: {} execution time: {}ms'.format(orig_func.__qualname__, run_time))
        global exec_time_record
        exec_time_record.append((orig_func.__qualname__, run_time))
        return result

    return wrapper


def print_ascii_art(text, font=pyfiglet.DEFAULT_FONT):
    print()
    print(pyfiglet.figlet_format(text=text, font=font))
    print()
